#include <math.h>
#include <stdio.h>
#include "raz.h"

/*
 *  raz.c: library of numerical integration routines
 *  Copyright (C) 1999  Marco Gaiarin <gaio@dei.unipd.it> and
 *			Jacopo Pietropolli Charmet <cico@dei.unipd.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * Questa libreria di funzioni C permette di calcolare l'integrale numerico
 * di una funziona di 5 variabili ``particolare'' (tempo, 2 variabili, di
 * tipo meccanico) o di 2 variabili generica, ovvero del tipo:
 *
 *   x[0] = t
 *   x[1] = x, x[2] = y
 *   x[3] = x', x[4] = y'
 *
 * e quindi il sistema:
 *
 *  / t'  = x[0]' = 1
 *  | x'  = x[1]' = x[3]
 *  < y'  = x[2]' = x[4]
 *  | x'' = x[3]' = f(x)
 *  \ y'' = x[4]' = g(x)
 *
 * Vengono implementati (con una interfaccia comune) i metodi di Eulero,
 * Eulero modificato, RungeKutta e RungeKuttaGill.
 */


double	xa[VARS];

/*	Metodo di integrazione di Eulero/1
 *
 *	Viene semplicemente calcolato il valore della derivata nel punto e
 *	calcolato il valore successivo su questa retta.
 */
int Eulero(double *x[VARS], double dt,
	double f(double[VARS]), double g(double[VARS]) ) {

double	rx[VARS] = {1, (*x)[3], (*x)[4], f(*x), g(*x) };
int	j;

for (j=0; j < VARS; j++)
	(*x)[j] = (*x)[j]+dt*rx[j];

return(0);

} // Eulero


/*	Metodo di integrazione di Eulero/2
 *
 *	Idem a sopra, solo che questa volta tutto  ricalcolato a partire
 *	dal punto di mezzo.
 */
int Eulero2(double *x[VARS], double dt,
	double f(double[VARS]), double g(double[VARS]) ) {

double	rx[VARS] = {1, (*x)[3], (*x)[4], f(*x), g(*x) };
int	i,
	j;

for (j=0; j < VARS; j++)
	xa[j] = (*x)[j]+0.5*dt*rx[j];

for (i = 0; i < 3; i++) {

  for (j=0; j < VARS; j++)
	(*x)[j] = xa[j]+0.5*dt*rx[j];

  rx[1] = (*x)[3];
  rx[2] = (*x)[4];
  rx[3] = f(*x);
  rx[4] = g(*x);

}

return(0);

} // Eulero2


/*	Metodo di integrazione di Runge-Kutta
 *
 *	Metodo del quarto ordine
 */
int RungeKutta(double *x[VARS], double dt,
	double f(double[VARS]), double g(double[VARS]) ) {

double	rx[VARS][4] = {	{1, 1, 1, 1},
			{(*x)[3], 1, 1, 1},
			{(*x)[4], 1, 1, 1},
			{f(*x), 1, 1, 1},
			{g(*x), 1, 1, 1} };

int	j;

for (j=0; j < VARS; j++)
  xa[j] = (*x)[j];

for (j=0; j < VARS; j++)
  (*x)[j] = xa[j]+0.5*dt*rx[j][0];

rx[1][1] = (*x)[3];
rx[2][1] = (*x)[4];
rx[3][1] = f(*x);
rx[4][1] = g(*x);
for (j=0; j < VARS; j++)
  (*x)[j] = xa[j]+0.5*dt*rx[j][1];

rx[1][1] = (*x)[3];
rx[2][1] = (*x)[4];
rx[3][2] = f(*x);
rx[4][2] = g(*x);
for (j=0; j < VARS; j++)
  (*x)[j] = xa[j]+dt*rx[j][2];

rx[1][1] = (*x)[3];
rx[2][1] = (*x)[4];
rx[3][3] = f(*x);
rx[4][3] = g(*x);
for (j=0; j < VARS; j++)
  (*x)[j] = xa[j]+dt*(rx[j][0]+rx[j][1]+rx[j][1]+rx[j][2]+rx[j][2]+rx[j][3])/6;

return(0);

} // RungeKutta


/*	Metodo di integrazione di Runge-Kutta-Gill
 *
 *	Metodo del quarto ordine
 */
int RungeKuttaGill(double *x[VARS], double dt,
	double f(double[VARS]), double g(double[VARS]) ) {

double	rx[VARS] = {1, 1, 1, 1, 1},
	rrx[VARS],
	aux[VARS],
	aa[4] = {S, Q, P, pow(6,-1)},
	bb[4] = {2, 1, 1, 2},
	cc[4] = {S, Q, P, S};
int	i,
	j;


if ((*x)[0] == 0)
  for (j=0; j < VARS; j++)
	xa[j] = 0;

for (i=0; i < 4; i++) {

  rx[1] = (*x)[3];
  rx[2] = (*x)[4];
  rx[3] = f(*x);
  rx[4] = g(*x);
  for (j=0; j < VARS; j++) {
    rrx[j] = rx[j]*dt;
    aux[j] = aa[i]*(rrx[j] - bb[i]*xa[j]);
    (*x)[j] = (*x)[j]+aux[j];
    xa[j] = xa[j] + 3*aux[j] - cc[i]*rrx[j];
  }
}

return(0);

} // RungeKuttaGill

